#include "appupdate.h"
#include <QPixmap>
#include <stdio.h>
#include <QLocale>
#include <QScrollBar>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonParseError>
using namespace  std;

#define JSON_FILE_PATH "/var/cache/kylin-system-updater/json/"

AppUpdateWid::AppUpdateWid(AppAllMsg msg,QWidget *parent):QWidget(parent)
{
    m_updateMutual = UpdateDbus::getInstance(this);
    qRegisterMetaType<AppMsg>("AppMsg"); //注册信号槽类型
    qRegisterMetaType<AppAllMsg>("AppAllMsg"); //注册信号槽类型
    appAllMsg = msg;
    updateAppUi(appAllMsg.name);
    this->setObjectName("AppUpdateWid");
    initConnect();
}

void AppUpdateWid::updateAppUi(QString name)
{
    this->setFixedHeight(60);
    appname.clear();
    appname = name;
    frameLayout = new QHBoxLayout();
    frameLayout->setContentsMargins(0,0,0,0);   //更新信息之间的间距
    frameLayout->setSpacing(0);
    this->setLayout(frameLayout);

    AppFrame = new QFrame(this);  //最外层窗口适配主题样式
    AppFrame->setFrameShape(QFrame::Box);
    AppFrame->setMaximumSize(QSize(16777215, 16777215));
    frameLayout->addWidget(AppFrame);

    mainVLayout = new QHBoxLayout();
//    smallHLayout = new QHBoxLayout();
    appTitleWid = new QWidget();

    iconNameLayout = new QHBoxLayout();
    appIconName = new QLabel(this);
    appIconName->setFixedWidth(200);
    appIcon = new QLabel(appIconName);
    appNameLab = new MyLabel(appIconName);
    appIconName->setLayout(iconNameLayout);
    appIcon->setFixedSize(32,32);
    iconNameLayout->addSpacing(6);
    iconNameLayout->addWidget(appIcon,1);
    iconNameLayout->addSpacing(8);
    iconNameLayout->addWidget(appNameLab,10);

    appVersion = new FixLabel(this);
    appVersion ->setFixedWidth(250);

    detaileInfo = new KBorderlessButton(this);
    //    detaileInfo->setText(tr("详情"));
    detaileInfo->setText(tr("details"));

    QPalette palette_datailInfo = detaileInfo->palette();
    palette_datailInfo.setColor(QPalette::ButtonText,QColor(55,144,250));
    detaileInfo->setPalette(palette_datailInfo);

    updateAPPBtn = new QPushButton(this);
    //    updateAPPBtn->setText(tr("更新"));
    updateAPPBtn->setText(tr("Update"));
    updateAPPBtn->setFixedWidth(95);
    updateAPPBtn->hide();

    mainVLayout->addWidget(appIconName,1,Qt::AlignLeft);
    mainVLayout->addWidget(appVersion,1,Qt::AlignLeft);
    mainVLayout->addWidget(detaileInfo,0,Qt::AlignHCenter);
    mainVLayout->addSpacing(60);
    mainVLayout->addWidget(updateAPPBtn,0,Qt::AlignHCenter);
    mainVLayout->setContentsMargins(0,0,10,0);
//    appTitleWid->setLayout(smallHLayout);
//    mainVLayout->addWidget(appTitleWid);
    mainVLayout->setMargin(0);
    mainVLayout->setSpacing(0);

    AppFrame->setLayout(mainVLayout);
    QMap<QString, QString> map = getNameAndIconFromJson(name);
    if (!map.value("name").isNull())
    {
        dispalyName = map.value("name");
        qInfo()<<"map.value.isNull()"<<dispalyName;
    }
    appNameLab->setText(dispalyName);
    /*判断图标，优先级: JSON文件指定 > qrc资源文件中 > 主题 > 默认*/
    if (!map.value("icon").isNull()) {
        haveThemeIcon = true;
        appIcon->setPixmap(QPixmap(map.value("icon")));
    } else if (name.contains("kylin-update-desktop-")||name == "linux-generic") {
        qInfo()<<"app exsist icon";
        haveThemeIcon = true;
        pkgIconPath = QString(":/img/upgrade/%1.png").arg(name);
        appIcon->setPixmap(QPixmap(pkgIconPath));
    } else if (QIcon::fromTheme(name).hasThemeIcon(name)) {    //判断是否有主题图标并输出
        haveThemeIcon = true;
        qInfo()<<"theme exsist icon";
        QIcon icon = QIcon::fromTheme(name);
        QPixmap pixmap = icon.pixmap(icon.actualSize(QSize(32, 32)));
        appIcon->setPixmap(pixmap);
    } else {
        qInfo()<<"else exsist icon";
        QIcon icon = QIcon::fromTheme("application-x-desktop");
        QPixmap pixmap = icon.pixmap(icon.actualSize(QSize(32, 32)));
        appIcon->setPixmap(pixmap);
    }
    QString newStrMsg = appAllMsg.availableVersion;

    if(newStrMsg.size()>16)
    {
        appVersion->setText(tr("Version:")+newStrMsg);
        appVersion->setToolTip(tr("Version:")+newStrMsg);
    }
    else
    {
        //        appVersion->setText(tr("最新：")+newStrMsg);
        appVersion->setText(tr("Version:")+newStrMsg);
        appVersion->setToolTip("");
    }
}


AppUpdateWid::~AppUpdateWid()
{
}

void AppUpdateWid::initConnect()
{
    connect(detaileInfo,&QPushButton::clicked,this,&AppUpdateWid::showDetails);
}

QMap<QString, QString> AppUpdateWid::getNameAndIconFromJson(QString pkgname)
{
    QMap <QString, QString> nameIconList;
    /*判断json文件是否存在*/
    QString filename = QString(JSON_FILE_PATH) +pkgname +".json";
    qInfo()<<"getNameAndIconFromJson's filename is "<<filename;
    QFile file(filename);
    if (!file.open(QIODevice::ReadOnly)){
        qInfo() << "JSON file open failed! ";
        file.close();
        return nameIconList;
    }
    QByteArray jsonData = file.readAll();

    QJsonParseError err_rpt;
    QJsonDocument  root_Doc = QJsonDocument::fromJson(jsonData, &err_rpt); // 字符串格式化为JSON

    if (!root_Doc.isNull() && (err_rpt.error == QJsonParseError::NoError)) {  // 解析未发生错误
        if (root_Doc.isObject()) { // JSON 文档为对象
            QJsonObject object = root_Doc.object();  // 转化为对象
            if (QLocale::system().name() == "zh_CN"){
                QString name  = object.value("name").toObject().value("zh_CN").toString();
                if (!name.isNull()) {
                    nameIconList.insert("name", name);
                }
            }else {
                QString name  = object.value("name").toObject().value("en_US").toString();
                if (!name.isNull()) {
                    nameIconList.insert("name", name);
                }
            }
            QString iconPath = object.value("icon").toString();
            if (!iconPath.isNull())
                nameIconList.insert("icon", iconPath);
        }
    }else{
        qInfo() << "JSON文件格式错误！";
        file.close();
        return nameIconList;
    }
    file.close();
    return nameIconList;
}

void AppUpdateWid::showDetails()
{
    qInfo()<<"show details in app";
    Detaildialog *detaillog=new Detaildialog();
    detaillog->setFixedSize(420,314);
    detaillog->logContent->clear();
    detaillog->setContentsMargins(24,24,24,24);
    QPoint detail_pos = detaileInfo->mapToGlobal(detaileInfo->pos());
    detaillog->move(QCursor().pos().x()-detaillog->width(),detail_pos.y()+detaileInfo->height());
    QString appname_lab = appNameLab->text();
    qInfo()<<"the appname_lab is"<< appname_lab;
    QString filename = "/var/cache/kylin-system-updater/json/"+appname+".json";
    qInfo()<<"file name is: "<<filename;
    QFile file(filename);
    if (!file.open(QIODevice::ReadOnly)){
        qInfo() << "JSON file open failed! "<<file.errorString();
        return ;
    }
    QByteArray jsonData = file.readAll();
    QJsonParseError err_rpt;
    QJsonDocument  root_Doc = QJsonDocument::fromJson(jsonData, &err_rpt); // 字符串格式化为JSON
    if (!root_Doc.isNull() && (err_rpt.error == QJsonParseError::NoError))
    {  // 解析未发生错误
        if (root_Doc.isObject())
        { // JSON 文档为对象
            QJsonObject object = root_Doc.object();  // 转化为对象
            if (QLocale::system().name() == "zh_CN")
            {
                QString name  = object.value("name").toObject().value("zh_CN").toString();
                QString description  = object.value("description").toObject().value("zh_CN").toString();
                if (!description.isNull()) {
                    detaillog->logContent->append(description);
//                    msg->shortDescription=description;
                }
                else
                {
                    detaillog->logContent->append(tr("Cumulative updates"));
                }
            }
            else
            {
                QString name  = object.value("name").toObject().value("en_US").toString();
                QString description  = object.value("description").toObject().value("en_US").toString();
                if (!description.isNull()) {
//                    msg->shortDescription=description;
                    detaillog->logContent->append(description);
                }
                else
                {
                    detaillog->logContent->append(tr("Cumulative updates"));
                }
            }
        }
    }
    file.close();
    detaillog->show();
}
